@@ -1,4 +1,4 @@
-{-| 
+{-|
 
 Hledger.Cli re-exports the options, utilities and commands provided by
 the hledger command-line program. This module also aggregates the
@@ -63,24 +63,25 @@ tests_Hledger_Cli = TestList
 
 
    ,"account directive" ~:
-   let sameParse str1 str2 = do j1 <- readJournal Nothing Nothing Nothing str1 >>= either error' return
-                                j2 <- readJournal Nothing Nothing Nothing str2 >>= either error' return
+   let ignoresourcepos j = j{jtxns=map (\t -> t{tsourcepos=nullsourcepos}) (jtxns j)} in
+   let sameParse str1 str2 = do j1 <- readJournal Nothing Nothing True Nothing str1 >>= either error' (return . ignoresourcepos)
+                                j2 <- readJournal Nothing Nothing True Nothing str2 >>= either error' (return . ignoresourcepos)
                                 j1 `is` j2{filereadtime=filereadtime j1, files=files j1, jContext=jContext j1}
    in TestList
    [
-    "account directive 1" ~: sameParse 
+    "account directive 1" ~: sameParse
                           "2008/12/07 One\n  test:from  $-1\n  test:to  $1\n"
                           "!account test\n2008/12/07 One\n  from  $-1\n  to  $1\n"
 
-   ,"account directive 2" ~: sameParse 
+   ,"account directive 2" ~: sameParse
                            "2008/12/07 One\n  test:foo:from  $-1\n  test:foo:to  $1\n"
                            "!account test\n!account foo\n2008/12/07 One\n  from  $-1\n  to  $1\n"
 
-   ,"account directive 3" ~: sameParse 
+   ,"account directive 3" ~: sameParse
                            "2008/12/07 One\n  test:from  $-1\n  test:to  $1\n"
                            "!account test\n!account foo\n!end\n2008/12/07 One\n  from  $-1\n  to  $1\n"
 
-   ,"account directive 4" ~: sameParse 
+   ,"account directive 4" ~: sameParse
                            ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" ++
                             "!account outer\n2008/12/07 Two\n  aigh  $-2\n  bee  $2\n" ++
                             "!account inner\n2008/12/07 Three\n  gamma  $-3\n  delta  $3\n" ++
@@ -95,7 +96,7 @@ tests_Hledger_Cli = TestList
                            )
 
    ,"account directive should preserve \"virtual\" posting type" ~: do
-      j <- readJournal Nothing Nothing Nothing "!account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return
+      j <- readJournal Nothing Nothing True Nothing "!account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return
       let p = head $ tpostings $ head $ jtxns j
       assertBool "" $ paccount p == "test:from"
       assertBool "" $ ptype p == VirtualPosting
@@ -103,7 +104,7 @@ tests_Hledger_Cli = TestList
    ]
 
    ,"account aliases" ~: do
-      j <- readJournal Nothing Nothing Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return
+      j <- readJournal Nothing Nothing True Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return
       let p = head $ tpostings $ head $ jtxns j
       assertBool "" $ paccount p == "equity:draw:personal:food"
 
@@ -125,7 +126,7 @@ tests_Hledger_Cli = TestList
   --     `is` "aa:aa:aaaaaaaaaaaaaa")
 
   ,"default year" ~: do
-    j <- readJournal Nothing Nothing Nothing defaultyear_journal_str >>= either error' return
+    j <- readJournal Nothing Nothing True Nothing defaultyear_journal_str >>= either error' return
     tdate (head $ jtxns j) `is` fromGregorian 2009 1 1
     return ()
 
@@ -135,7 +136,7 @@ tests_Hledger_Cli = TestList
 
  ]
 
-  
+
 -- fixtures/test data
 
 -- date1 = parsedate "2008/11/26"
@@ -339,9 +340,10 @@ defaultyear_journal_str = unlines
 --  ]
 
 journal7 :: Journal
-journal7 = nulljournal {jtxns = 
+journal7 = nulljournal {jtxns =
           [
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/01",
              tdate2=Nothing,
              tstatus=False,
@@ -357,6 +359,7 @@ journal7 = nulljournal {jtxns =
            }
           ,
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/02/01",
              tdate2=Nothing,
              tstatus=False,
@@ -372,6 +375,7 @@ journal7 = nulljournal {jtxns =
            }
           ,
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/02",
              tdate2=Nothing,
              tstatus=False,
@@ -387,6 +391,7 @@ journal7 = nulljournal {jtxns =
            }
           ,
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
              tstatus=False,
@@ -402,6 +407,7 @@ journal7 = nulljournal {jtxns =
            }
           ,
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
              tstatus=False,
@@ -417,6 +423,7 @@ journal7 = nulljournal {jtxns =
            }
           ,
            txnTieKnot Transaction {
+             tsourcepos=nullsourcepos,
              tdate=parsedate "2007/01/03",
              tdate2=Nothing,
              tstatus=False,
