@@ -20,10 +20,10 @@ import Safe (headDef, headMay)
 import System.Console.CmdArgs.Explicit
 import System.Console.Haskeline (runInputT, defaultSettings, setComplete)
 import System.Console.Haskeline.Completion
-import System.Console.Wizard  
+import System.Console.Wizard
 import System.Console.Wizard.Haskeline
 import System.IO ( stderr, hPutStr, hPutStrLn )
-import Text.ParserCombinators.Parsec hiding (Line)
+import Text.Parsec
 import Text.Printf
 
 import Hledger
@@ -128,9 +128,9 @@ confirmedTransactionWizard es@EntryState{..} = do
   -- liftIO $ hPrintf stderr {- "Transaction entered:\n%s" -} (show t)
   output $ show t
   y <- let def = "y" in
-       retryMsg "Please enter y or n." $ 
-        parser ((fmap ('y' ==)) . headMay . map toLower . strip) $ 
-        defaultTo' def $ nonEmpty $ 
+       retryMsg "Please enter y or n." $
+        parser ((fmap ('y' ==)) . headMay . map toLower . strip) $
+        defaultTo' def $ nonEmpty $
         maybeRestartTransaction $
         line $ green $ printf "Save this transaction to the journal ?%s: " (showDefault def)
   if y then return t else throw RestartTransactionException
@@ -167,10 +167,10 @@ similarTransaction EntryState{..} desc =
 
 dateAndCodeWizard EntryState{..} = do
   let def = headDef (showDate esDefDate) esArgs
-  retryMsg "A valid hledger smart date is required. Eg: 2014/2/14, 14, yesterday." $ 
-   parser (parseSmartDateAndCode esToday) $ 
+  retryMsg "A valid hledger smart date is required. Eg: 2014/2/14, 14, yesterday." $
+   parser (parseSmartDateAndCode esToday) $
    withCompletion (dateCompleter def) $
-   defaultTo' def $ nonEmpty $ 
+   defaultTo' def $ nonEmpty $
    maybeExit $
    maybeRestartTransaction $
    -- maybeShowHelp $
@@ -178,7 +178,8 @@ dateAndCodeWizard EntryState{..} = do
     where
       parseSmartDateAndCode refdate s = either (const Nothing) (\(d,c) -> return (fixSmartDate refdate d, c)) edc
           where
-            edc = parseWithCtx nullctx dateandcodep $ lowercase s
+            edc = runParser dateandcodep nullctx "" $ lowercase s
+            dateandcodep :: Stream [Char] m t => ParsecT [Char] JournalContext m (SmartDate, String)
             dateandcodep = do
                 d <- smartdate
                 c <- optionMaybe codep
@@ -191,7 +192,7 @@ dateAndCodeWizard EntryState{..} = do
 descriptionAndCommentWizard EntryState{..} = do
   let def = headDef "" esArgs
   s <- withCompletion (descriptionCompleter esJournal def) $
-       defaultTo' def $ nonEmpty $ 
+       defaultTo' def $ nonEmpty $
        maybeRestartTransaction $
        line $ green $ printf "Description%s: " (showDefault def)
   let (desc,comment) = (strip a, strip $ dropWhile (==';') b) where (a,b) = break (==';') s
@@ -213,7 +214,7 @@ postingWizard es@EntryState{..} = do
     let es1 = es{esArgs=drop 1 esArgs}
     (amt,comment)  <- amountAndCommentWizard es1
     return $ Just nullposting{paccount=stripbrackets acct
-                             ,pamount=mixed amt
+                             ,pamount=Mixed [amt]
                              ,pcomment=comment
                              ,ptype=accountNamePostingType acct
                              }
@@ -233,7 +234,7 @@ accountWizard EntryState{..} = do
   retryMsg "A valid hledger account name is required. Eg: assets:cash, expenses:food:eating out." $
    parser (parseAccountOrDotOrNull def canfinish) $
    withCompletion (accountCompleter esJournal def) $
-   defaultTo' def $ -- nonEmpty $ 
+   defaultTo' def $ -- nonEmpty $
    maybeRestartTransaction $
    line $ green $ printf "Account %d%s%s: " pnum (endmsg::String) (showDefault def)
     where
@@ -241,7 +242,7 @@ accountWizard EntryState{..} = do
       parseAccountOrDotOrNull _  _ "."       = dbg $ Just "." -- . always signals end of txn
       parseAccountOrDotOrNull "" True ""     = dbg $ Just ""  -- when there's no default and txn is balanced, "" also signals end of txn
       parseAccountOrDotOrNull def@(_:_) _ "" = dbg $ Just def -- when there's a default, "" means use that
-      parseAccountOrDotOrNull _ _ s          = dbg $ either (const Nothing) validateAccount $ parseWithCtx (jContext esJournal) accountnamep s -- otherwise, try to parse the input as an accountname
+      parseAccountOrDotOrNull _ _ s          = dbg $ either (const Nothing) validateAccount $ runParser accountnamep (jContext esJournal) "" s -- otherwise, try to parse the input as an accountname
       dbg = id -- strace
       validateAccount s | no_new_accounts_ esOpts && not (s `elem` journalAccountNames esJournal) = Nothing
                         | otherwise = Just s
@@ -259,14 +260,15 @@ amountAndCommentWizard EntryState{..} = do
               _  | pnum > 1 && not (isZeroMixedAmount balancingamt) -> showamt balancingamt
               _                                                     -> ""
   retryMsg "A valid hledger amount is required. Eg: 1, $2, 3 EUR, \"4 red apples\"." $
-   parser parseAmountAndComment $ 
+   parser parseAmountAndComment $
    withCompletion (amountCompleter def) $
-   defaultTo' def $ nonEmpty $ 
+   defaultTo' def $ nonEmpty $
    maybeRestartTransaction $
    line $ green $ printf "Amount  %d%s: " pnum (showDefault def)
-    where  
-      parseAmountAndComment = either (const Nothing) Just . parseWithCtx nodefcommodityctx amountandcommentp
-      nodefcommodityctx = (jContext esJournal){ctxCommodityAndStyle=Nothing}
+    where
+      parseAmountAndComment = either (const Nothing) Just . runParser amountandcommentp nodefcommodityctx ""
+      nodefcommodityctx = (jContext esJournal){ctxDefaultCommodityAndStyle=Nothing}
+      amountandcommentp :: Stream [Char] m t => ParsecT [Char] JournalContext m (Amount, String)
       amountandcommentp = do
         a <- amountp
         many spacenonewline
@@ -298,7 +300,7 @@ maybeExit = parser (\s -> if s=="." then throw UnexpectedEOF else Just s)
 maybeRestartTransaction = parser (\s -> if s=="<" then throw RestartTransactionException else Just s)
 
 -- maybeShowHelp :: Wizard Haskeline String -> Wizard Haskeline String
--- maybeShowHelp wizard = maybe (liftIO showHelp >> wizard) return $ 
+-- maybeShowHelp wizard = maybe (liftIO showHelp >> wizard) return $
 --                        parser (\s -> if s=="?" then Nothing else Just s) wizard
 
 -- Completion helpers
