@@ -1,4 +1,5 @@
 {-# LANGUAGE ScopedTypeVariables, DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleContexts #-}
 {-|
 
 Common cmdargs modes and flags, a command-line options type, and
@@ -10,8 +11,10 @@ module Hledger.Cli.Options (
 
   -- * cmdargs flags & modes
   helpflags,
+  detailedversionflag,
   inputflags,
   reportflags,
+  outputflags,
   generalflagsgroup1,
   generalflagsgroup2,
   generalflagsgroup3,
@@ -21,7 +24,7 @@ module Hledger.Cli.Options (
   argsFlag,
   showModeHelp,
   withAliases,
-  
+
   -- * CLI options
   CliOpts(..),
   defcliopts,
@@ -29,6 +32,8 @@ module Hledger.Cli.Options (
   decodeRawOpts,
   rawOptsToCliOpts,
   checkCliOpts,
+  outputFormats,
+  defaultOutputFormat,
 
   -- possibly these should move into argsToCliOpts
   -- * CLI option accessors
@@ -36,14 +41,17 @@ module Hledger.Cli.Options (
   aliasesFromOpts,
   journalFilePathFromOpts,
   rulesFilePathFromOpts,
+  outputFileFromOpts,
+  outputFormatFromOpts,
   -- | For register:
   OutputWidth(..),
   Width(..),
   defaultWidth,
   defaultWidthWithFlag,
   widthFromOpts,
+  maybeAccountNameDrop,
   -- | For balance:
-  formatFromOpts,
+  lineFormatFromOpts,
 
   -- * Other utils
   hledgerAddons,
@@ -51,9 +59,10 @@ module Hledger.Cli.Options (
   -- * Tests
   tests_Hledger_Cli_Options
 
-) 
+)
 where
-  
+
+import Control.Applicative ((<$>))
 import qualified Control.Exception as C
 import Control.Monad (when)
 import Data.List
@@ -67,7 +76,7 @@ import System.Environment
 import System.Exit (exitSuccess)
 import System.FilePath
 import Test.HUnit
-import Text.ParserCombinators.Parsec as P
+import Text.Parsec
 
 import Hledger
 import Hledger.Data.OutputFormat as OutputFormat
@@ -81,16 +90,21 @@ helpflags :: [Flag RawOpts]
 helpflags = [
   flagNone ["help","h"] (setboolopt "help") "show general help or (after command) command help"
  -- ,flagNone ["browse-args"] (setboolopt "browse-args") "use a web UI to select options and build up a command line"
- ,flagOpt "1" ["debug"] (\s opts -> Right $ setopt "debug" s opts) "N" "show debug output (increase N for more)"
+ ,flagReq  ["debug"]    (\s opts -> Right $ setopt "debug" s opts) "N" "show debug output if N is 1-9 (default: 0)"
  ,flagNone ["version"] (setboolopt "version") "show version information"
  ]
 
+-- | A hidden flag, just for the hledger executable.
+detailedversionflag :: Flag RawOpts
+detailedversionflag = flagNone ["version+"] (setboolopt "version+") "show version information with extra detail"
+
 -- | Common input-related flags: --file, --rules-file, --alias...
 inputflags :: [Flag RawOpts]
 inputflags = [
   flagReq ["file","f"]  (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use -"
  ,flagReq ["rules-file"]  (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
  ,flagReq ["alias"]  (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "display accounts named OLD as NEW"
+ ,flagNone ["ignore-assertions"] (setboolopt "ignore-assertions") "ignore any balance assertions in the journal"
  ]
 
 -- | Common report-related flags: --period, --cost, etc.
@@ -98,22 +112,28 @@ reportflags :: [Flag RawOpts]
 reportflags = [
   flagReq  ["begin","b"]     (\s opts -> Right $ setopt "begin" s opts) "DATE" "include postings/txns on or after this date"
  ,flagReq  ["end","e"]       (\s opts -> Right $ setopt "end" s opts) "DATE" "include postings/txns before this date"
- ,flagNone ["daily","D"]     (\opts -> setboolopt "daily" opts) "multiperiod/multicolumn report by day"
- ,flagNone ["weekly","W"]    (\opts -> setboolopt "weekly" opts) "multiperiod/multicolumn report by week"
- ,flagNone ["monthly","M"]   (\opts -> setboolopt "monthly" opts) "multiperiod/multicolumn report by month"
- ,flagNone ["quarterly","Q"] (\opts -> setboolopt "quarterly" opts) "multiperiod/multicolumn report by quarter"
- ,flagNone ["yearly","Y"]    (\opts -> setboolopt "yearly" opts) "multiperiod/multicolumn report by year"
+ ,flagNone ["daily","D"]     (setboolopt "daily") "multiperiod/multicolumn report by day"
+ ,flagNone ["weekly","W"]    (setboolopt "weekly") "multiperiod/multicolumn report by week"
+ ,flagNone ["monthly","M"]   (setboolopt "monthly") "multiperiod/multicolumn report by month"
+ ,flagNone ["quarterly","Q"] (setboolopt "quarterly") "multiperiod/multicolumn report by quarter"
+ ,flagNone ["yearly","Y"]    (setboolopt "yearly") "multiperiod/multicolumn report by year"
  ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or reporting interval all at once (overrides the flags above)"
- ,flagNone ["date2","aux-date"] (\opts -> setboolopt "date2" opts) "use postings/txns' secondary dates instead"
+ ,flagNone ["date2","aux-date"] (setboolopt "date2") "use postings/txns' secondary dates instead"
 
- ,flagNone ["cleared","C"]   (\opts -> setboolopt "cleared" opts) "include only cleared postings/txns"
- ,flagNone ["uncleared","U"] (\opts -> setboolopt "uncleared" opts) "include only uncleared postings/txns"
- ,flagNone ["real","R"]      (\opts -> setboolopt "real" opts) "include only non-virtual postings"
+ ,flagNone ["cleared","C"]   (setboolopt "cleared") "include only pending/cleared postings/txns"
+ ,flagNone ["uncleared","U"] (setboolopt "uncleared") "include only uncleared postings/txns"
+ ,flagNone ["real","R"]      (setboolopt "real") "include only non-virtual postings"
  ,flagReq  ["depth"]         (\s opts -> Right $ setopt "depth" s opts) "N" "hide accounts/postings deeper than N"
- ,flagNone ["empty","E"]     (\opts -> setboolopt "empty" opts) "show empty/zero things which are normally omitted"
- ,flagNone ["cost","B"]      (\opts -> setboolopt "cost" opts) "show amounts in their cost price's commodity"
+ ,flagNone ["empty","E"]     (setboolopt "empty") "show empty/zero things which are normally omitted"
+ ,flagNone ["cost","B"]      (setboolopt "cost") "show amounts in their cost price's commodity"
  ]
 
+-- | Common output-related flags: --output-file, --output-format...
+outputflags = [
+   flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE[.FMT]" "write output to FILE instead of stdout. A recognised FMT suffix influences the format."
+  ,flagReq  ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT" "select the output format. Supported formats: txt, csv."
+  ]
+
 argsFlag :: FlagHelp -> Arg RawOpts
 argsFlag desc = flagArg (\s opts -> Right $ setopt "args" s opts) desc
 
@@ -198,7 +218,7 @@ standardAddonsHelp = [
 
 -- | Get a mode's help message as a nicely wrapped string.
 showModeHelp :: Mode a -> String
-showModeHelp = (showText defaultWrap :: [Text] -> String) . 
+showModeHelp = (showText defaultWrap :: [Text] -> String) .
                (helpText [] HelpFormatDefault :: Mode a -> [Text])
 
 -- | Add command aliases to the command's help string.
@@ -229,7 +249,10 @@ data CliOpts = CliOpts {
     ,command_         :: String
     ,file_            :: Maybe FilePath
     ,rules_file_      :: Maybe FilePath
+    ,output_file_     :: Maybe FilePath
+    ,output_format_   :: Maybe String
     ,alias_           :: [String]
+    ,ignore_assertions_ :: Bool
     ,debug_           :: Int            -- ^ debug level, set by @--debug[=N]@. See also 'Hledger.Utils.debugLevel'.
     ,no_new_accounts_ :: Bool           -- add
     ,width_           :: Maybe String   -- register
@@ -249,10 +272,13 @@ defcliopts = CliOpts
     def
     def
     def
+    def
+    def
+    def
 
 -- | Convert possibly encoded option values to regular unicode strings.
 decodeRawOpts :: RawOpts -> RawOpts
-decodeRawOpts = map (\(name,val) -> (name, fromSystemString val))
+decodeRawOpts = map (\(name',val) -> (name', fromSystemString val))
 
 -- | Parse raw option string values to the desired final data types.
 -- Any relative smart dates will be converted to fixed dates based on
@@ -265,17 +291,20 @@ rawOptsToCliOpts rawopts = do
              ,command_         = stringopt "command" rawopts
              ,file_            = maybestringopt "file" rawopts
              ,rules_file_      = maybestringopt "rules-file" rawopts
+             ,output_file_     = maybestringopt "output-file" rawopts
+             ,output_format_   = maybestringopt "output-format" rawopts
              ,alias_           = map stripquotes $ listofstringopt "alias" rawopts
              ,debug_           = intopt "debug" rawopts
+             ,ignore_assertions_ = boolopt "ignore-assertions" rawopts
              ,no_new_accounts_ = boolopt "no-new-accounts" rawopts -- add
              ,width_           = maybestringopt "width" rawopts    -- register
              ,reportopts_      = ropts
              }
-  
+
 -- | Do final validation of processed opts, raising an error if there is trouble.
 checkCliOpts :: CliOpts -> IO CliOpts -- or pure..
 checkCliOpts opts@CliOpts{reportopts_=ropts} = do
-  case formatFromOpts ropts of
+  case lineFormatFromOpts ropts of
     Left err -> optserror $ "could not parse format option: "++err
     Right _ -> return ()
   case widthFromOpts opts of
@@ -288,26 +317,26 @@ checkCliOpts opts@CliOpts{reportopts_=ropts} = do
 -- cmdargs mode, and either return them or, if a help flag is present,
 -- print the mode help and exit the program.
 getCliOpts :: Mode RawOpts -> IO CliOpts
-getCliOpts mode = do
-  args <- getArgs
-  let rawopts = decodeRawOpts $ processValue mode args
+getCliOpts mode' = do
+  args' <- getArgs
+  let rawopts = decodeRawOpts $ processValue mode' args'
   opts <- rawOptsToCliOpts rawopts >>= checkCliOpts
-  debugArgs args opts
+  debugArgs args' opts
   -- if any (`elem` args) ["--help","-h","-?"]
   when ("help" `inRawOpts` rawopts_ opts) $
-    putStr (showModeHelp mode) >> exitSuccess
+    putStr (showModeHelp mode') >> exitSuccess
   return opts
   where
     -- | Print debug info about arguments and options if --debug is present.
     debugArgs :: [String] -> CliOpts -> IO ()
-    debugArgs args opts =
-      when ("--debug" `elem` args) $ do
-        progname <- getProgName
-        putStrLn $ "running: " ++ progname
-        putStrLn $ "raw args: " ++ show args
+    debugArgs args' opts =
+      when ("--debug" `elem` args') $ do
+        progname' <- getProgName
+        putStrLn $ "running: " ++ progname'
+        putStrLn $ "raw args: " ++ show args'
         putStrLn $ "processed opts:\n" ++ show opts
         d <- getCurrentDay
-        putStrLn $ "search query: " ++ (show $ queryFromOpts d $ reportopts_ opts)
+        putStrLn $ "search query: " ++ show (queryFromOpts d $ reportopts_ opts)
 
 -- CliOpts accessors
 
@@ -332,6 +361,44 @@ journalFilePathFromOpts opts = do
   d <- getCurrentDirectory
   expandPath d $ fromMaybe f $ file_ opts
 
+
+-- | Get the expanded, absolute output file path from options,
+-- or the default (-, meaning stdout).
+outputFileFromOpts :: CliOpts -> IO FilePath
+outputFileFromOpts opts = do
+  d <- getCurrentDirectory
+  case output_file_ opts of
+    Just p  -> expandPath d p
+    Nothing -> return "-"
+
+defaultOutputFormat = "txt"
+
+outputFormats =
+  [defaultOutputFormat] ++
+  ["csv"
+  ]
+
+-- | Get the output format from the --output-format option,
+-- otherwise from a recognised file extension in the --output-file option,
+-- otherwise the default (txt).
+outputFormatFromOpts :: CliOpts -> String
+outputFormatFromOpts opts =
+  case output_format_ opts of
+    Just f  -> f
+    Nothing ->
+      case filePathExtension <$> output_file_ opts of
+        Just ext | ext `elem` outputFormats -> ext
+        _                                   -> defaultOutputFormat
+
+-- -- | Get the file name without its last extension, from a file path.
+-- filePathBaseFileName :: FilePath -> String
+-- filePathBaseFileName = fst . splitExtension . snd . splitFileName
+
+-- | Get the last file extension, without the dot, from a file path.
+-- May return the null string.
+filePathExtension :: FilePath -> String
+filePathExtension = dropWhile (=='.') . snd . splitExtension . snd . splitFileName
+
 -- | Get the (tilde-expanded) rules file path from options, if any.
 rulesFilePathFromOpts :: CliOpts -> IO (Maybe FilePath)
 rulesFilePathFromOpts opts = do
@@ -342,12 +409,12 @@ rulesFilePathFromOpts opts = do
 
 -- | Parse the format option if provided, possibly returning an error,
 -- otherwise get the default value.
-formatFromOpts :: ReportOpts -> Either String [OutputFormat]
-formatFromOpts = maybe (Right defaultBalanceFormat) parseStringFormat . format_
+lineFormatFromOpts :: ReportOpts -> Either String [OutputFormat]
+lineFormatFromOpts = maybe (Right defaultBalanceLineFormat) parseStringFormat . format_
 
 -- | Default line format for balance report: "%20(total)  %2(depth_spacer)%-(account)"
-defaultBalanceFormat :: [OutputFormat]
-defaultBalanceFormat = [
+defaultBalanceLineFormat :: [OutputFormat]
+defaultBalanceLineFormat = [
       FormatField False (Just 20) Nothing TotalField
     , FormatLiteral "  "
     , FormatField True (Just 2) Nothing DepthSpacerField
@@ -358,7 +425,7 @@ defaultBalanceFormat = [
 
 -- | Output width configuration (for register).
 data OutputWidth =
-    TotalWidth Width    -- ^ specify the overall width 
+    TotalWidth Width    -- ^ specify the overall width
   | FieldWidths [Width] -- ^ specify each field's width
   deriving Show
 
@@ -388,17 +455,22 @@ parseWidth s = case (runParser outputwidthp () "(unknown)") s of
     Left  e -> Left $ show e
     Right x -> Right x
 
-outputwidthp :: GenParser Char st OutputWidth
+outputwidthp :: Stream [Char] m t => ParsecT [Char] st m OutputWidth
 outputwidthp =
   try (do w <- widthp
           ws <- many1 (char ',' >> widthp)
           return $ FieldWidths $ w:ws)
   <|> TotalWidth `fmap` widthp
 
-widthp :: GenParser Char st Width
+widthp :: Stream [Char] m t => ParsecT [Char] st m Width
 widthp = (string "auto" >> return Auto)
     <|> (Width . read) `fmap` many1 digit
 
+-- | Drop leading components of accounts names as specified by --drop, but only in --flat mode.
+maybeAccountNameDrop :: ReportOpts -> AccountName -> AccountName
+maybeAccountNameDrop opts a | tree_ opts = a
+                            | otherwise  = accountNameDrop (drop_ opts) a
+
 -- Other utils
 
 -- | Get the sorted unique precise names and display names of hledger
@@ -422,8 +494,7 @@ hledgerAddons = do
                      -- groupBy (\a b -> dropExtension a == dropExtension b) $
                      map stripprefix exes
   let displaynames = concatMap stripext $
-                     groupBy (\a b -> dropExtension a == dropExtension b) $
-                     precisenames
+                     groupBy (\a b -> dropExtension a == dropExtension b) precisenames
   return (precisenames, displaynames)
   where
     stripprefix = drop (length progname + 1)
@@ -456,9 +527,9 @@ isHledgerExeName :: String -> Bool
 isHledgerExeName = isRight . parsewith hledgerexenamep
     where
       hledgerexenamep = do
-        string progname
-        char '-'
-        many1 (noneOf ".")
+        _ <- string progname
+        _ <- char '-'
+        _ <- many1 (noneOf ".")
         optional (string "." >> choice' (map string addonExtensions))
         eof
 
